package caclient;

import gov.aps.jca.Channel;
import gov.aps.jca.Context;
import gov.aps.jca.JCALibrary;
import gov.aps.jca.Monitor;
import gov.aps.jca.dbr.DBR;
import gov.aps.jca.dbr.DBRType;
import gov.aps.jca.dbr.DBR_Double;
import gov.aps.jca.event.ConnectionEvent;
import gov.aps.jca.event.ConnectionListener;
import gov.aps.jca.event.MonitorEvent;
import gov.aps.jca.event.MonitorListener;

/** Demo class: Subscribes to 'double', printing every value.
 *  @author Kay Kasemir
 */
public class CAMonitorDouble implements ConnectionListener, MonitorListener
{
    /** The Java CA Library instance. Should be app-wide singleton. */
    static private JCALibrary jca = null;

    /** The JCA context. Should probably be app-wide singleton. */
    static private Context jca_context = null;

    /** Ref count for the singletons */
    static private int jca_refs = 0;

    /** Channel reference. */
    private Channel channel = null;

    /** Did we already subscribe? */
    private boolean subscribed = false;

    /** Constructor based on channel name. */
    public CAMonitorDouble(String name) throws Exception
    {
        if (jca_refs == 0)
        {
            System.out.println("Creating JCA context");
            jca = JCALibrary.getInstance();
            // This one line picks the 'pure java' implementation.
            // The alternative (JNI_...) requires system-specific JNI binaries.
            jca_context = jca.createContext(JCALibrary.CHANNEL_ACCESS_JAVA);
        }
        ++jca_refs;
        channel = jca_context.createChannel(name);
        channel.addConnectionListener(this);
        // Note: doesn't handle the special case that the channel is
        //       immediately connected, because we already used that PV name!
        
        jca_context.flushIO();
        // Note:
        // When dealing with more than one channel, it's advisable to
        // issue several 'createChannel()' calls, and then add a single, final
        // flushIO, since this allows the CA client lib to pool the search
        // requests into fewer network packets.
    }
    
    /** Dispose this channel. */
    public void dispose() throws Exception
    {
        channel.dispose();
        --jca_refs;
        if (jca_refs > 0)
            return;
        System.out.println("Disposing JCA context");
        jca_context.destroy();
        jca = null;
    }
    
    /** ConnectionListener interface. */
    public void connectionChanged(ConnectionEvent ev)
    {
        try
        {
            if (ev.isConnected())
            {
                System.out.println("Connected to " + channel.getName());
                if (subscribed)
                    return; // don't subscribe more than once
                // We subscribe to DOUBLE.
                // Should use <native type> ...
                int count = channel.getElementCount();
                channel.addMonitor(DBRType.DOUBLE, count, Monitor.VALUE, this);
                jca_context.flushIO();
                // Note:
                // Instead of flushing right away, it's better to set a 
                // "I need to flush" flag and then handle that periodically.
                // That way, the client lib. can again optimize the network
                // packages.
            }
            else
                System.out.println("Disconnected from " + channel.getName());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    
    /** MonitorListener interface. */
    public void monitorChanged(MonitorEvent ev)
    {
        if (ev.getStatus().isSuccessful())
        {
            DBR dbr = ev.getDBR();
            double v[] = ((DBR_Double)dbr).getDoubleValue();
            System.out.print(channel.getName());
            for (int i = 0; i < v.length; i++)
                if (i == v.length-1)
                    System.out.println(" " + v[i]);
                else
                    System.out.print(" " + v[i]);
        }
        else
            System.out.println(channel.getName() + "':"
                    + ev.getStatus().getMessage());
    }
}
